package com.situm.situm_flutter

import android.util.Log
import es.situm.sdk.SitumSdk
import es.situm.sdk.directions.DirectionsRequest
import es.situm.sdk.error.Error
import es.situm.sdk.location.LocationListener
import es.situm.sdk.location.LocationStatus
import es.situm.sdk.model.cartography.Building
import es.situm.sdk.model.directions.Route
import es.situm.sdk.model.location.Location
import es.situm.sdk.model.navigation.NavigationProgress
import es.situm.sdk.navigation.NavigationListener
import es.situm.sdk.navigation.NavigationRequest
import es.situm.sdk.utils.Handler
import io.flutter.plugin.common.MethodChannel

class Navigation private constructor(
) : LocationListener, NavigationListener {

    private lateinit var channel: MethodChannel
    private lateinit var osHandler: android.os.Handler

    // Navigation will be a singleton so we can be sure that calls to
    // locationManager.addListener(this) always receive the same instance.
    companion object {
        private var instance: Navigation? = null

        fun init(channel: MethodChannel, handler: android.os.Handler): Navigation {
            if (instance == null) {
                instance = Navigation()
            }
            instance?.let {
                it.channel = channel
                it.osHandler = handler
            }
            return instance!!
        }
    }

    fun request(
        buildingIdentifier: String,
        directionsRequestArgs: Map<String, Any>,
        navigationRequestArgs: Map<String, Any>?,
        result: MethodChannel.Result,
    ) {
        SitumSdk.navigationManager().removeUpdates()
        // Directions handler: receive the calculated route.
        val directionsHandler = object : Handler<Route> {
            override fun onSuccess(route: Route) {
                if (navigationRequestArgs != null) {
                    // If requested, start navigation:
                    val navigationRequest = NavigationRequest.fromMap(navigationRequestArgs)
                    navigationRequest.route = route
                    SitumSdk.navigationManager().requestNavigationUpdates(
                        navigationRequest, this@Navigation
                    )
                }
                // Both requestDirections and requestNavigation methods will return the calculated
                // route.
                result.success(route.toMap())
            }

            override fun onFailure(error: Error) {
                result.notifySitumSdkError(error)
            }
        }
        // Handler for fetchBuilding: DirectionsRequest mapper requires a Building object to create
        // indoor points so we start calling fetchBuilding().
        val buildingHandler = object : Handler<Building> {
            override fun onSuccess(building: Building) {
                val directionsRequest = DirectionsRequest.fromMap(directionsRequestArgs)
                SitumSdk.directionsManager().requestDirections(directionsRequest, directionsHandler)
            }

            override fun onFailure(error: Error) {
                result.notifySitumSdkError(error)
            }
        }
        SitumSdk.communicationManager().fetchBuilding(buildingIdentifier, buildingHandler)
        // Add this class as location listener to keep navigationManager up to date:
        SitumSdk.locationManager().addLocationListener(this)
    }

    // Location listener:

    override fun onLocationChanged(location: Location) {
        if (SitumSdk.navigationManager().isRunning) {
            SitumSdk.navigationManager().updateWithLocation(location)
        }
    }

    override fun onStatusChanged(status: LocationStatus) {
        // TODO!
    }

    override fun onError(error: Error) {
        SitumSdk.navigationManager().removeUpdates()
    }

    // Navigation listener:

    override fun onDestinationReached() {
        osHandler.post {
            channel.invokeMethod("onNavigationFinished", null)
        }
    }

    override fun onProgress(progress: NavigationProgress) {
        osHandler.post {
            channel.invokeMethod("onNavigationProgress", progress.toMap())
        }
    }

    override fun onUserOutsideRoute() {
        osHandler.post {
            channel.invokeMethod("onUserOutsideRoute", null)
        }
    }
}